@@ -37,7 +37,11 @@ import sys
 import tempfile
 import unittest
 from vsc.utils import fancylogger
-from vsc.utils.fancylogger import setLogLevelDebug
+
+# disable all logging to significantly speed up tests
+import easybuild.tools.build_log  # initialize EasyBuild logging, so we disable it
+fancylogger.disableDefaultHandlers()
+fancylogger.setLogLevelError()
 
 # toolkit should be first to allow hacks to work
 import test.framework.asyncprocess as a
@@ -88,7 +92,6 @@ os.close(fd)
 os.remove(log_fn)
 fancylogger.logToFile(log_fn)
 log = fancylogger.getLogger()
-setLogLevelDebug()
 
 # call suite() for each module and then run them all
 # note: make sure the options unit tests run first, to avoid running some of them with a readily initialized config
