@@ -32,8 +32,9 @@ from unittest import TestLoader, main
 
 from easybuild.tools.systemtools import AMD, ARM, DARWIN, INTEL, LINUX, UNKNOWN
 from easybuild.tools.systemtools import get_avail_core_count, get_core_count
-from easybuild.tools.systemtools import get_cpu_model, get_cpu_speed, get_cpu_vendor
-from easybuild.tools.systemtools import get_os_type, get_shared_lib_ext, get_platform_name, get_os_name, get_os_version
+from easybuild.tools.systemtools import get_cpu_model, get_cpu_speed, get_cpu_vendor, get_glibc_version
+from easybuild.tools.systemtools import get_os_type, get_os_name, get_os_version, get_platform_name, get_shared_lib_ext
+from easybuild.tools.systemtools import get_system_info
 
 
 class SystemToolsTest(EnhancedTestCase):
@@ -94,6 +95,16 @@ class SystemToolsTest(EnhancedTestCase):
         os_version = get_os_version()
         self.assertTrue(isinstance(os_version, basestring) or os_version == UNKNOWN)
 
+    def test_glibc_version(self):
+        """Test getting glibc version."""
+        glibc_version = get_glibc_version()
+        self.assertTrue(isinstance(glibc_version, basestring) or glibc_version == UNKNOWN)
+
+    def test_system_info(self):
+        """Test getting system info."""
+        system_info = get_system_info()
+        self.assertTrue(isinstance(system_info, dict))
+
 def suite():
     """ returns all the testcases in this module """
     return TestLoader().loadTestsFromTestCase(SystemToolsTest)
